#include <amxmodx>
#include <amxmisc>
#include <fakemeta_util>
#include <hamsandwich>
#include <engine>
#include <xs>
#include <nvault>

#define PLUGIN	"HD Sprays Lite"
#define VERSION	"1.2"
#define AUTHOR	"trofian"

#define play_spray_sound(%1)		emit_sound(%1, CHAN_AUTO, "player/sprayer.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
#define is_entity_player(%1)		(1<=%1<=g_maxplayers)
#define is_valid_spray(%1)		(0<=%1<g_TotalSprays)
#define xs_1_neg(%1)			%1 = -%1
#define SPRAYS_PATH			"models/next21_sprays"

#define VIP_FLAGS			ADMIN_LEVEL_H
#define MAX_SPRAYS_LOAD			20	// максимальное кол-во спреев, которое может быть в загружено
#define MAX_SPRAYS_ALIVE		120	// максимальное кол-во спреев, которое может одновременно существовать на карте
#define	TIME_ERASE_SPRAY		200.0	// время в сек., через которое спрей удалится
#define TIME_SET_SPRAY			100.0	// время в сек., через которое снова можно поставить спрей
#define PRESS_DELAY			0.2	// задержка для использования кнопки t

#define	NVAULT_DB        	"next21_spray_config"

//---------------------------------------------<Настройки wall checker>-----------------------
#define WALL_CHECKER_DEBUG_LEVEL 	0

#define	HARD_CALC				0 // более точная проверка поверхности для рисования спрея (может положить сервер)
#define SURFACE_CHECK_NUM		5
#define ADDITIONAL_DIVIDER		2.0

#define ADD_UNITS			2.0
#define USABLE_DIST			130.0	// максимальная дистанция на которой работает спрей

new
g_SprayMdl[MAX_SPRAYS_LOAD][128], g_SprayName[MAX_SPRAYS_LOAD][128],
g_SprayVip[MAX_SPRAYS_LOAD], g_SprayLoopAnim[MAX_SPRAYS_LOAD],
Float:g_SprayWidth[MAX_SPRAYS_LOAD], Float:g_SprayHeight[MAX_SPRAYS_LOAD],

g_maxplayers,
g_SpraysFile[128], g_TotalSprays, g_Menu, g_Sprays_Now,

g_PlayerSpray[33],/* не id энтитии, а id спрея */ Float:g_Player_Last_Set[33], Float:g_Player_Press_Timeout[33],
g_PlayerFlags[33],

g_info_target, 

g_Vault,

g_pcvar_is_light

#if WALL_CHECKER_DEBUG_LEVEL > 0
new g_iBeamSprite
#endif

public plugin_cfg()
{
	g_Vault = nvault_open(NVAULT_DB)
			
	if(g_Vault == INVALID_HANDLE)
		set_fail_state("Error opening nVault!")
		
	nvault_prune(g_Vault, 0, get_systime() - (86400 * 30))
}

public plugin_precache()
{
	new cfgDir[32]
	get_configsdir(cfgDir,31)
	formatex(g_SpraysFile,63,"%s/HD_Sprays.ini",cfgDir)
	command_load()
	
	for (new i; i < g_TotalSprays; i++)
	{
		new CurrFile[256]
		formatex(CurrFile, charsmax(CurrFile), "%s/%s", SPRAYS_PATH, g_SprayMdl[i])
			
		if(file_exists(CurrFile))
		{
			precache_model(CurrFile)
			server_print("[%s] Precached %s",PLUGIN,CurrFile)
			format(g_SprayMdl[i], 127, "%s/%s", SPRAYS_PATH, g_SprayMdl[i])
		}
		else
			server_print("[%s] Failed to precache %s",PLUGIN,CurrFile)
	}
	
	#if WALL_CHECKER_DEBUG_LEVEL > 0
	g_iBeamSprite = precache_model("sprites/laserbeam.spr");
	#endif
}

public command_load()
{
	if(!file_exists(g_SpraysFile))
		return
		
	new
	sfLineData[512],
	file = fopen(g_SpraysFile,"rt"),
	sWidth[16], sHeight[16], sVip[16], sLoopAnim[16],
	sI[11], text[256]
	
	g_Menu = menu_create("\ySprays Menu", "sprays_menu_handler")
	menu_additem(g_Menu, "Убрать спрей", "1337", 0)
	
	while(file && !feof(file))
	{
		fgets(file, sfLineData, charsmax(sfLineData))
			
		if (containi(sfLineData,";") > -1)
			continue
		
		parse(sfLineData, g_SprayMdl[g_TotalSprays], 127, g_SprayName[g_TotalSprays], 127, sWidth, charsmax(sWidth), sHeight, charsmax(sHeight), sVip, charsmax(sVip), sLoopAnim, charsmax(sLoopAnim))
		g_SprayWidth[g_TotalSprays] = str_to_float(sWidth)
		g_SprayHeight[g_TotalSprays] = str_to_float(sHeight)
		
		if(equal(sVip, "yes"))
			g_SprayVip[g_TotalSprays] = 1
		
		if(equal(sLoopAnim, "yes"))
			g_SprayLoopAnim[g_TotalSprays] = 1
		
		num_to_str(g_TotalSprays, sI, charsmax(sI))
		if(g_SprayVip[g_TotalSprays])
			formatex(text, charsmax(text), "%s \r(VIP)", g_SprayName[g_TotalSprays])
		else
			formatex(text, charsmax(text), "%s", g_SprayName[g_TotalSprays])
		menu_additem(g_Menu, text, sI, 0)
		
		g_TotalSprays++
		if(g_TotalSprays >= MAX_SPRAYS_LOAD)
		{
			server_print("[%s] Reached spray limit", PLUGIN)
			break
		}
	}
	
	if(file)
		fclose(file)
	
	menu_setprop(g_Menu, MPROP_NEXTNAME, "Вперёд")
	menu_setprop(g_Menu, MPROP_BACKNAME, "Назад")
	menu_setprop(g_Menu, MPROP_EXITNAME, "Выход")
	menu_setprop(g_Menu, MPROP_EXIT, MEXIT_ALL)
	
	if(g_TotalSprays == 1)
		server_print("[%s] Loaded one spray", PLUGIN)
	else
		server_print("[%s] Loaded %i sprays", PLUGIN, g_TotalSprays)
}

public plugin_init()
{
	register_plugin(PLUGIN, VERSION, AUTHOR)
	RegisterHam(Ham_Spawn, "player", "HAM_Player_Spawn_post", 1)
	RegisterHam(Ham_Killed, "player", "HAM_Player_Killed_post", 1)
	register_event("HLTV", "HLTV", "a", "1=0", "2=0")
	
	register_clcmd("say /spray", "show_sprays_menu")
	register_clcmd("say_team /spray", "show_sprays_menu")
	
	g_pcvar_is_light = register_cvar("hd_spray_light","0")
	
	g_maxplayers = get_maxplayers()
	g_info_target = engfunc(EngFunc_AllocString, "info_target")
	register_impulse(201, "hook_real_spray")
}

public plugin_end()
{
	nvault_close(g_Vault)
}

public hook_real_spray(id)
{
	if(!is_valid_spray(g_PlayerSpray[id]))
		return PLUGIN_CONTINUE
	
	if(get_gametime() < g_Player_Press_Timeout[id])
		return PLUGIN_HANDLED
	
	g_Player_Press_Timeout[id] = get_gametime()+PRESS_DELAY
	create_spray(id)
	
	return PLUGIN_HANDLED
}

public create_spray(id)
{
	if(!is_user_alive(id))
		return
	
	if(!is_valid_spray(g_PlayerSpray[id]))
		return
	
	if(g_Player_Last_Set[id] > get_gametime())
		return
	
	new Float:fOrigin[3], Float:fAngles[3]
	if(!get_corrected_origin(id, fOrigin, g_SprayWidth[g_PlayerSpray[id]], g_SprayHeight[g_PlayerSpray[id]], fAngles))
		return
	
	if(g_Sprays_Now > MAX_SPRAYS_ALIVE)
		return
	
	play_spray_sound(id)
	
	g_Sprays_Now++
	g_Player_Last_Set[id] = get_gametime() + TIME_SET_SPRAY
	vector_to_angle(fAngles, fAngles)
	
	
	if(fAngles[0]==90.0||fAngles[0]==270.0){
		new Float:vAngles[3];
		pev(id,pev_v_angle,vAngles);
		fAngles[1]=vAngles[1]-180.0;
	}
	
	new spray = engfunc(EngFunc_CreateNamedEntity, g_info_target)
	set_pev(spray, pev_classname, "next21_spray")
	engfunc(EngFunc_SetModel, spray, g_SprayMdl[g_PlayerSpray[id]])
	set_pev(spray, pev_solid, SOLID_TRIGGER)
	set_pev(spray, pev_movetype, MOVETYPE_FLY)
	set_pev(spray, pev_origin, fOrigin)
	
	set_pev(spray, pev_angles, fAngles)

	set_task(TIME_ERASE_SPRAY, "erase_spray", spray)
	
	if(get_pcvar_num(g_pcvar_is_light))
		set_pev(spray, pev_effects, EF_DIMLIGHT)
	
	if(!g_SprayLoopAnim[g_PlayerSpray[id]])
		return
	
	set_pev(spray, pev_framerate, 1.0)
	set_pev(spray, pev_sequence, 0)
	set_pev(spray, pev_animtime, fm_halflife_time())
}

public erase_spray(spray)
{
	if(pev_valid(spray))
		engfunc(EngFunc_RemoveEntity, spray)
	
	g_Sprays_Now--
}

public show_sprays_menu(id)
{
	menu_display(id, g_Menu, 0)
	
	return PLUGIN_HANDLED
}

public sprays_menu_handler(id, menu, item)
{
	if(item == MENU_EXIT)
		return PLUGIN_HANDLED
	
	new s_Data[6], s_Name[64], i_Access, i_Callback
	menu_item_getinfo(menu, item, i_Access, s_Data, charsmax(s_Data), s_Name, charsmax(s_Name), i_Callback)
	
	new key
	key = str_to_num(s_Data)
	
	if(is_valid_spray(key) && g_SprayVip[key] && ~g_PlayerFlags[id] & VIP_FLAGS)
	{
		menu_display(id, g_Menu, 0)
		client_print(id, print_center, "Этот спрей только для VIP")
		return PLUGIN_HANDLED
	}
	
	g_PlayerSpray[id] = key
	
	new szKey[35], szSpraySettings[2]
	get_user_authid(id, szKey, 34)
	szSpraySettings[0] = key
	
	nvault_set(g_Vault, szKey, szSpraySettings)
	
	return PLUGIN_CONTINUE
}

public HAM_Player_Spawn_post(id)
{
	new iFlags = get_user_flags(id)
			
	if (g_PlayerFlags[id] != iFlags)
	{
		g_PlayerFlags[id] = iFlags
		load_spray_settings(id)
	}
}

public HAM_Player_Killed_post(vic, att)
{
	if(is_entity_player(att) && is_valid_spray(g_PlayerSpray[att]))
		g_Player_Last_Set[att] = 0.0
}

public client_putinserver(id)
	g_PlayerSpray[id] = 1337

public HLTV()
{
	for(new i=1; i<=g_maxplayers; i++)
		g_Player_Last_Set[i] = 0.0
}


// wall checker
public get_corrected_origin(id, Float:fOutPointFSet[3], Float:fWidth, Float:fHeight, Float:fOrigWallNormal[3])
{
	new Float:fWallNormal[3]
	get_wall_normal(id, fWallNormal)
	xs_vec_copy(fWallNormal, fOrigWallNormal)
	
	new Float:fWallAngles[3]
	vector_to_angle(fWallNormal, fWallAngles)
	
	new Float:fUpNormal[3], Float:fRightNormal[3]
	angle_vector(fWallAngles, ANGLEVECTOR_UP, fUpNormal)
	angle_vector(fWallAngles, ANGLEVECTOR_RIGHT, fRightNormal)
	
	new Float:fAimOrigin[3], Float:fPlSOrigin[3], Float:fBetween
	fm_get_aim_origin(id, fAimOrigin)
	pev(id, pev_origin, fPlSOrigin)
	fBetween = floatsqroot(floatpower(fPlSOrigin[0]-fAimOrigin[0], 2.0) + floatpower(fPlSOrigin[1]-fAimOrigin[1], 2.0) + floatpower(fPlSOrigin[2]-fAimOrigin[2], 2.0))
	if(fBetween > USABLE_DIST)
		return 0
	
	xs_vec_copy(fAimOrigin, fOutPointFSet)
	
	new Float:fAddUnits[3]
	xs_vec_mul_scalar(fOrigWallNormal, ADD_UNITS, fAddUnits)
	xs_vec_add(fAddUnits, fAimOrigin, fAimOrigin)
	
	xs_1_neg(fUpNormal[2])
	
	new Float:fUpLeftPoint[3], Float:fUpRightPoint[3], Float:fDownLeftPoint[3], Float:fDownRightPoint[3]
	xs_vec_mul_scalar(fUpNormal, fHeight/2, fUpNormal)
	xs_vec_mul_scalar(fRightNormal, fWidth/2, fRightNormal)
	
	//1
	xs_vec_add(fUpNormal, fRightNormal, fUpRightPoint)
	xs_vec_add(fUpRightPoint, fAimOrigin, fUpRightPoint)
	
	//2
	xs_vec_neg(fRightNormal, fRightNormal)
	xs_vec_add(fUpNormal, fRightNormal, fUpLeftPoint)
	xs_vec_add(fUpLeftPoint, fAimOrigin, fUpLeftPoint)
	
	//3
	xs_vec_neg(fUpNormal, fUpNormal)
	xs_vec_add(fUpNormal, fRightNormal, fDownLeftPoint)
	xs_vec_add(fDownLeftPoint, fAimOrigin, fDownLeftPoint)
	
	//4
	xs_vec_neg(fRightNormal, fRightNormal)
	xs_vec_add(fUpNormal, fRightNormal, fDownRightPoint)
	xs_vec_add(fDownRightPoint, fAimOrigin, fDownRightPoint)
	//
	
	xs_vec_neg(fWallNormal, fWallNormal)
	
	new isUpRightPoint, isUpLeftPoint, isDownLeftPoint, isDownRightPoint
	isUpRightPoint = trace_to_wall(fUpRightPoint, fWallNormal)
	isUpLeftPoint = trace_to_wall(fUpLeftPoint, fWallNormal)
	isDownLeftPoint = trace_to_wall(fDownLeftPoint, fWallNormal)
	isDownRightPoint = trace_to_wall(fDownRightPoint, fWallNormal)
	
	if(!isUpRightPoint || !isUpLeftPoint || !isDownLeftPoint || !isDownRightPoint)
		return 0
	
	#if HARD_CALC > 0
	if(isUpRightPoint && isUpLeftPoint && isDownLeftPoint && isDownRightPoint){	}
	else if(isUpRightPoint && isUpLeftPoint && !isDownLeftPoint && !isDownRightPoint)
	{
		new Float:check_step = (fHeight/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		xs_vec_normalize(fUpNormal, fUpNormal)	// down
		xs_vec_neg(fUpNormal, fUpNormal)		// up
		
		if(!check_wall_in_a_cycle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fUpNormal, fWallNormal, check_step))
			return 0
	}
	else if(!isUpRightPoint && !isUpLeftPoint && isDownLeftPoint && isDownRightPoint)
	{
		new Float:check_step = (fHeight/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		xs_vec_normalize(fUpNormal, fUpNormal)	// down
		
		if(!check_wall_in_a_cycle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fUpNormal, fWallNormal, check_step))
			return 0
	}
	else if(isUpRightPoint && isDownRightPoint && !isUpLeftPoint && !isDownLeftPoint)
	{
		new Float:check_step = (fWidth/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		xs_vec_normalize(fRightNormal, fRightNormal)	// right
		
		if(!check_wall_in_a_cycle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fRightNormal, fWallNormal, check_step))
			return 0
	}
	else if(!isUpRightPoint && !isDownRightPoint && isUpLeftPoint && isDownLeftPoint)
	{
		new Float:check_step = (fWidth/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		xs_vec_normalize(fRightNormal, fRightNormal)	// right
		xs_vec_neg(fRightNormal, fRightNormal)			// left
		
		check_wall_in_a_cycle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fRightNormal, fWallNormal, check_step)
	}
	else if((!isUpRightPoint && isDownRightPoint && !isUpLeftPoint && !isDownLeftPoint) ||
			(isUpRightPoint && isDownRightPoint && !isUpLeftPoint && isDownLeftPoint))
	{
		new Float:check_step_n1 = (fWidth/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		new Float:check_step_n2 = (fHeight/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		
		xs_vec_normalize(fRightNormal, fRightNormal)	// right
		xs_vec_normalize(fUpNormal, fUpNormal)			// down
		
		if(!check_wall_in_a_cycle_double(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fRightNormal, fUpNormal, fWallNormal, check_step_n1, check_step_n2))
			return 0
	}
	else if((!isUpRightPoint && !isDownRightPoint && !isUpLeftPoint && isDownLeftPoint) ||
			(!isUpRightPoint && isDownRightPoint && isUpLeftPoint && isDownLeftPoint))
	{
		new Float:check_step_n1 = (fWidth/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		new Float:check_step_n2 = (fHeight/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		
		xs_vec_normalize(fRightNormal, fRightNormal)	// right
		xs_vec_neg(fRightNormal, fRightNormal)			// left
		xs_vec_normalize(fUpNormal, fUpNormal)			// down
		
		if(!check_wall_in_a_cycle_double(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fRightNormal, fUpNormal, fWallNormal, check_step_n1, check_step_n2))
			return 0
	}
	else if((!isUpRightPoint && !isDownRightPoint && isUpLeftPoint && !isDownLeftPoint) ||
			(isUpRightPoint && !isDownRightPoint && isUpLeftPoint && isDownLeftPoint))
	{
		new Float:check_step_n1 = (fWidth/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		new Float:check_step_n2 = (fHeight/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		
		xs_vec_normalize(fRightNormal, fRightNormal)	// right
		xs_vec_neg(fRightNormal, fRightNormal)			// left
		xs_vec_normalize(fUpNormal, fUpNormal)			// down
		xs_vec_neg(fUpNormal, fUpNormal)				// up
		
		if(!check_wall_in_a_cycle_double(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fRightNormal, fUpNormal, fWallNormal, check_step_n1, check_step_n2))
			return 0
	}
	else if((isUpRightPoint && !isDownRightPoint && !isUpLeftPoint && !isDownLeftPoint) ||
			(isUpRightPoint && isDownRightPoint && isUpLeftPoint && !isDownLeftPoint))
	{
		new Float:check_step_n1 = (fWidth/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		new Float:check_step_n2 = (fHeight/ADDITIONAL_DIVIDER)/SURFACE_CHECK_NUM
		
		xs_vec_normalize(fRightNormal, fRightNormal)	// right
		xs_vec_normalize(fUpNormal, fUpNormal)			// down
		xs_vec_neg(fUpNormal, fUpNormal)				// up
		
		if(!check_wall_in_a_cycle_double(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fRightNormal, fUpNormal, fWallNormal, check_step_n1, check_step_n2))
			return 0
	}
	#endif
	
	#if WALL_CHECKER_DEBUG_LEVEL > 0
	create_rectangle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, 'g', 50)
	#endif
	
	angle_vector(fWallAngles, ANGLEVECTOR_UP, fUpNormal)
	angle_vector(fWallAngles, ANGLEVECTOR_RIGHT, fRightNormal)
	
	xs_1_neg(fUpNormal[2])
	
	xs_vec_mul_scalar(fRightNormal, fWidth/2, fRightNormal)
	xs_vec_mul_scalar(fUpNormal, fHeight/2, fUpNormal)
	
	new Float:fTmpVec[3]
	xs_vec_add(fDownLeftPoint, fRightNormal, fTmpVec)
	xs_vec_add(fTmpVec, fUpNormal, fOutPointFSet)
	
	return 1
}

#if HARD_CALC > 0
check_wall_in_a_cycle(Float:fUpRightPoint[3], Float:fUpLeftPoint[3], Float:fDownLeftPoint[3], Float:fDownRightPoint[3],
	Float:fNormal[3], Float:fWallNormal[3], Float:check_step)
{
	new Float:fTmpVec[3]
	xs_vec_mul_scalar(fNormal, check_step, fTmpVec)
	
	for(new i=1; i<=SURFACE_CHECK_NUM; i++)
	{
		xs_vec_add(fUpRightPoint, fTmpVec, fUpRightPoint)
		xs_vec_add(fUpLeftPoint, fTmpVec, fUpLeftPoint)
		xs_vec_add(fDownLeftPoint, fTmpVec, fDownLeftPoint)
		xs_vec_add(fDownRightPoint, fTmpVec, fDownRightPoint)
		
		if(check_wall(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fWallNormal))
		{
			#if WALL_CHECKER_DEBUG_LEVEL == 2
			create_rectangle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, 'r', 50)
			#endif
			return 1
		}
		
		#if WALL_CHECKER_DEBUG_LEVEL == 2
		create_rectangle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, 'b', 25)
		#endif
	}
	
	return 0
}

check_wall_in_a_cycle_double(Float:fUpRightPoint[3], Float:fUpLeftPoint[3], Float:fDownLeftPoint[3], Float:fDownRightPoint[3],
	Float:fNormal1[3], Float:fNormal2[3], Float:fWallNormal[3], Float:check_step1, Float:check_step2)
{
	new Float:fTmpVec1[3]
	xs_vec_mul_scalar(fNormal1, check_step1, fTmpVec1)
	
	new Float:fTmpVec2[3]
	xs_vec_mul_scalar(fNormal2, check_step2, fTmpVec2)
	
	for(new i=1; i<=SURFACE_CHECK_NUM; i++)
	{
		xs_vec_add(fUpRightPoint, fTmpVec1, fUpRightPoint)
		xs_vec_add(fUpRightPoint, fTmpVec2, fUpRightPoint)
		xs_vec_add(fUpLeftPoint, fTmpVec1, fUpLeftPoint)
		xs_vec_add(fUpLeftPoint, fTmpVec2, fUpLeftPoint)
		xs_vec_add(fDownLeftPoint, fTmpVec1, fDownLeftPoint)
		xs_vec_add(fDownLeftPoint, fTmpVec2, fDownLeftPoint)
		xs_vec_add(fDownRightPoint, fTmpVec1, fDownRightPoint)
		xs_vec_add(fDownRightPoint, fTmpVec2, fDownRightPoint)
		
		if(check_wall(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, fWallNormal))
		{
			#if WALL_CHECKER_DEBUG_LEVEL == 2
			create_rectangle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, 'r', 50)
			#endif
			return 1
		}
		
		#if WALL_CHECKER_DEBUG_LEVEL == 2
		create_rectangle(fUpRightPoint, fUpLeftPoint, fDownLeftPoint, fDownRightPoint, 'b', 25)
		#endif
	}
	
	return 0
}

check_wall(Float:fUpRightPoint[3], Float:fUpLeftPoint[3], Float:fDownLeftPoint[3], Float:fDownRightPoint[3], Float:fWallNormal[3])
{
	if(
	trace_to_wall(fUpRightPoint, fWallNormal)	&&
	trace_to_wall(fUpLeftPoint, fWallNormal)	&&
	trace_to_wall(fDownLeftPoint, fWallNormal)	&&
	trace_to_wall(fDownRightPoint, fWallNormal)) return 1
	
	return 0
}
#endif

get_wall_normal(id, Float:fNormal[3])
{
	new Float:fOrigin[3]
	pev(id, pev_origin, fOrigin)
	
	new Float:fAngles[3]
	pev(id, pev_v_angle, fAngles)
	angle_vector(fAngles, ANGLEVECTOR_FORWARD, fAngles)
	xs_vec_mul_scalar(fAngles, 9999.0, fAngles)
	
	new Float:fEndPos[3]
	xs_vec_add(fAngles, fOrigin, fEndPos)
	
	new ptr = create_tr2()	
	engfunc(EngFunc_TraceLine, fOrigin, fEndPos, IGNORE_MISSILE | IGNORE_MONSTERS | IGNORE_GLASS, id, ptr)
	
	new Float:vfNormal[3]
	get_tr2(ptr, TR_vecPlaneNormal, vfNormal)
	
	free_tr2(ptr)
	
	xs_vec_copy(vfNormal, fNormal)
}

trace_to_wall(Float:fOrigin[3], Float:fVec[3])
{
	new Float:fOrigin2[3]
	xs_vec_mul_scalar(fVec, ADD_UNITS, fOrigin2)
	xs_vec_add(fOrigin2, fOrigin, fOrigin2)
	xs_vec_add(fOrigin2, fVec, fOrigin2)
	
	new ptr = create_tr2()
	engfunc(EngFunc_TraceLine, fOrigin, fOrigin2, IGNORE_MISSILE | IGNORE_MONSTERS | IGNORE_GLASS, 0, ptr)
	
	new Float:fFrac
	get_tr2(ptr, TR_flFraction, fFrac)
	
	free_tr2(ptr)
	
	if(fFrac == 1.0)
		return 0
	
	return 1
}

load_spray_settings(id)
{
	new szKey[35], szSpraySettings[3], iTimestamp
	get_user_authid(id, szKey, 34)
	if (nvault_lookup(g_Vault, szKey, szSpraySettings, 2, iTimestamp))
	{
		if (!is_valid_spray(szSpraySettings[0]))
		{
			szSpraySettings[0] = 1337
		}
		else if (g_SprayVip[szSpraySettings[0]] && ~g_PlayerFlags[id] & VIP_FLAGS)
		{
			szSpraySettings[0] = 1337
			nvault_set(g_Vault, szKey, szSpraySettings)
		}
		
		g_PlayerSpray[id] = szSpraySettings[0]
	}	
}

#if WALL_CHECKER_DEBUG_LEVEL > 0
create_rectangle(Float:fUpRightPoint[3], Float:fUpLeftPoint[3], Float:fDownLeftPoint[3], Float:fDownRightPoint[3], color, lifetime)
{
	new r,g,b
	switch(color)
	{
		case 'r': r=255
		case 'g': g=255
		case 'b': b=255
	}
	
	set_sprite(fUpRightPoint, fUpLeftPoint, r, g, b, lifetime)
	set_sprite(fUpLeftPoint, fDownLeftPoint, r, g, b, lifetime)
	set_sprite(fDownLeftPoint, fDownRightPoint, r, g, b, lifetime)
	set_sprite(fDownRightPoint, fUpRightPoint, r, g, b, lifetime)
}

set_sprite(Float:fOrigin[3], Float:fOrigin2[3], r, g, b, lifetime)
{
	message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
	write_byte(TE_BEAMPOINTS)
	engfunc(EngFunc_WriteCoord, fOrigin[0])
	engfunc(EngFunc_WriteCoord, fOrigin[1])
	engfunc(EngFunc_WriteCoord, fOrigin[2])
	engfunc(EngFunc_WriteCoord, fOrigin2[0])
	engfunc(EngFunc_WriteCoord, fOrigin2[1])
	engfunc(EngFunc_WriteCoord, fOrigin2[2])
	write_short(g_iBeamSprite)
	write_byte(0)
	write_byte(1)
	write_byte(lifetime)
	write_byte(20)
	write_byte(0)
	write_byte(r)
	write_byte(g)
	write_byte(b)
	write_byte(200)
	write_byte(0)
	message_end()
}
#endif
